package org.kropp.examples;

import java.util.List;

import net.sf.xanswer.indexing.IndexException;

import org.kropp.index.Index;
import org.kropp.index.Query;
import org.kropp.index.AbstractQuery;
import org.kropp.index.Condition;
import org.kropp.index.impl.SearchEngine;
import org.kropp.util.xml.XMLIndexFactory;

/**
 * Example of constructing own queries
 * 
 * @author Victor Kropp
 */
public class SearchDummyQuery {
	/**
	 * Searches given xml file for hard-coded dummy query. This is example of
	 * constructing your own queries.
	 * 
	 * @param args
	 *            Command-line arguments.
	 */
	public static void main(String[] args) throws IndexException {
		if (args.length != 1) {
			System.err.println("Usage: java " + new CurrentClassGetter().getName()
					+ " <XML file>");
			System.exit(1);
		}

		Index index = XMLIndexFactory.getInstance().openIndex(args[0], "");

		try {
			/*
			 * Creating query.
			 */
			Query dummyQuery = new AbstractQuery() {
				{
					addCondition(new Condition(Query.Axis.CHILD, "project", ""));
					addCondition(new Condition(Query.Axis.CHILD, "description", ""));
				}
			};

			/*
			 * Making search.
			 */
			List<String> result = SearchEngine.search(index, dummyQuery);
			System.out.println(result);
		} catch (Exception e) {
			System.err.println("Some error occured.");
		}
	}

	private static class CurrentClassGetter extends SecurityManager {
		public String getName() {
			return getClassContext()[1].getName();
		}
	}
}
