package org.kropp.index.impl;

import net.sf.xanswer.indexing.IndexException;

import org.dbxml.core.DBException;
import org.dbxml.core.data.Value;
import org.dbxml.core.indexer.IndexQuery;
import org.exist.storage.store.*;
import org.exist.util.*;
import org.kropp.index.*;

import java.io.File;
import java.util.*;

/**
 * @author Victor Kropp
 */
public class IndexImpl implements Index {
	private String myFilename;

	private String myDirectory;

	private BFile myParentIndex;

	private BFile myPreorderIndex;

	private BFile myPostorderIndex;

	private BFile myData;

	private boolean myIsLoaded = false;

	public IndexImpl(String filename) {
		this(filename, "");
	}

	public IndexImpl(String filename, String directory) {
		myDirectory = directory;
		myFilename = filename;
	}

	public void create(String filename) throws IndexException {
		myFilename = filename;
		try {
			myParentIndex = new BFile(new File(myDirectory + filename + ".par"));
			myPreorderIndex = new BFile(new File(myDirectory + filename
					+ ".pre"));
			myPostorderIndex = new BFile(new File(myDirectory + filename
					+ ".post"));
			myData = new BFile(new File(myDirectory + filename + ".data"));

			myParentIndex.create();
			myPreorderIndex.create();
			myPostorderIndex.create();
			myData.create();
		} catch (Exception e) {
			throw new IndexException(e);
		}
	}

	public void open() throws IndexException {
		try {
			File parentFile = new File(myDirectory + myFilename + ".par");
			File preorderFile = new File(myDirectory + myFilename + ".pre");
			File postorderFile = new File(myDirectory + myFilename + ".post");
			File dataFile = new File(myDirectory + myFilename + ".data");

			if (!parentFile.exists() || !preorderFile.exists()
					|| !postorderFile.exists() || !dataFile.exists()) {
				throw new IndexException("Index doesn't exist.");
			}

			myParentIndex = new BFile(parentFile);
			myPreorderIndex = new BFile(preorderFile);
			myPostorderIndex = new BFile(postorderFile);
			myData = new BFile(dataFile);

			myParentIndex.open();
			myPreorderIndex.open();
			myPostorderIndex.open();
			myData.open();

			myIsLoaded = true;
		} catch (Exception e) {
			throw new IndexException(e);
		}
	}

	public boolean isLoaded() {
		return myIsLoaded;
	}

	public void add(IndexNode node) {
		try {
			List cur = myParentIndex.findValues(new IndexQuery(node
					.getParentKey()));
			if (cur.size() > 0) {
				Value curval = (Value) cur.get(0);
				byte[] newval = new byte[curval.getData().length + 4];
				System.arraycopy(curval.getData(), 0, newval, 0, curval
						.getData().length);
				ByteConversion.intToByte(node.getPreorder(), newval,
						newval.length - 4);
				myParentIndex.update(node.getParentKey(), new FixedByteArray(
						newval));
			} else {
				myParentIndex.append(node.getParentKey(), new FixedByteArray(
						ByteConversion.intToByte(node.getPreorder())));
			}

			myPreorderIndex.append(node.getPreorderKey(), node
					.getNodeNameByteArray());
			myPostorderIndex.append(node.getPostorderKey(), new FixedByteArray(
					ByteConversion.intToByte(node.getPreorder())));
			myData.append(node.getPreorderKey(), node.getValue());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String getFilename() {
		return myFilename;
	}

	@Override
	public String toString() {
		return new Integer(myPreorderIndex.getFixedKeyLen()).toString();
	}

	public BFile getPostorderIndex() {
		try {
			myPostorderIndex.flush();
		} catch (Exception e) {
		}
		return myPostorderIndex;
	}

	public BFile getPreorderIndex() {
		try {
			myPreorderIndex.flush();
		} catch (Exception e) {
		}
		return myPreorderIndex;
	}

	public BFile getParentIndex() {
		try {
			myParentIndex.flush();
		} catch (Exception e) {
		}
		return myParentIndex;
	}

	public BFile getDataFile() {
		try {
			myData.flush();
		} catch (Exception e) {
		}
		return myData;
	}

	public void flush() {
		try {
			myParentIndex.flush();
			myPreorderIndex.flush();
			myPostorderIndex.flush();
			myData.flush();
		} catch (DBException dbe) {
		}
	}

	public void close() throws IndexException {
		try {
			myParentIndex.close();
			myPreorderIndex.close();
			myPostorderIndex.close();
			myData.close();
		} catch (DBException dbe) {
			throw new IndexException(dbe);
		}
	}

	public void delete() {
		try {
			close();
		} catch (IndexException e) {
		}

		myParentIndex.getFile().delete();
		myPreorderIndex.getFile().delete();
		myPostorderIndex.getFile().delete();
		myData.getFile().delete();
	}
}
