package org.kropp.index.impl;

import org.dbxml.core.data.Value;
import org.exist.util.*;

/**
 * Index node.
 * 
 * @author Victor Kropp
 */
public class IndexNodeImpl implements org.kropp.index.IndexNode {
	private int myPreorder;

	private int myPostorder;

	private int myParent;

	private boolean myIsAttribute;

	private String myNodeName;

	private StringBuffer myNodeValue;

	public IndexNodeImpl(int preorder, int postorder, int parent,
			boolean attribute, String nodeName, String nodeValue) {
		myPreorder = preorder;
		myPostorder = postorder;
		myParent = parent;
		myIsAttribute = attribute;
		myNodeName = nodeName;
		myNodeValue = new StringBuffer(nodeValue);
	}

	public static IndexNodeImpl fromValue(Value val) {
		//byte[] data = new byte[ba.size()];
		//ba.copyTo(data, 0);
		byte[] data = val.getData();

		int offset = 21 + ByteConversion.byteToInt(data, 17);
		
		return new IndexNodeImpl(ByteConversion.byteToInt(data, 0),
				ByteConversion.byteToInt(data, 4), ByteConversion.byteToInt(
						data, 8), (data[12] == 1) ? true : false,
				ByteConversion.UTF82String(data, 21, ByteConversion.byteToInt(
						data, 13)), (offset == data.length) ? "" : ByteConversion.UTF82String(data,
						offset + 4, ByteConversion.byteToInt(data, offset)));
	}

	public boolean isAttribute() {
		return myIsAttribute;
	}

	public String getNodeName() {
		return myNodeName;
	}

	public String getNodeValue() {
		return myNodeValue.toString();
	}

	public void appendNodeValue(String text) {
		myNodeValue.append(text);
	}

	public int getParent() {
		return myParent;
	}

	public int getPostorder() {
		return myPostorder;
	}

	public int getPreorder() {
		return myPreorder;
	}

	public void setPostorder(int postorder) {
		myPostorder = postorder;
	}

	public ByteArray getValue() {
		byte[] byteNodeName = ByteConversion.string2UTF8(myNodeName);
		byte[] byteNodeValue = ByteConversion.string2UTF8(myNodeValue
				.toString());
		byte[] data = new byte[25 + byteNodeName.length + byteNodeValue.length];

		ByteConversion.intToByte(myPreorder, data, 0);
		ByteConversion.intToByte(myPostorder, data, 4);
		ByteConversion.intToByte(myParent, data, 8);
		data[12] = (byte) (myIsAttribute ? 1 : 0);
		ByteConversion.intToByte(myNodeName.length(), data, 13);
		ByteConversion.intToByte(byteNodeName.length, data, 17);
		System.arraycopy(byteNodeName, 0, data, 21, byteNodeName.length);
		ByteConversion.intToByte(myNodeValue.length(), data,
				21 + byteNodeName.length);
		System.arraycopy(byteNodeValue, 0, data, 25 + byteNodeName.length,
				byteNodeValue.length);

		return new FixedByteArray(data);
	}
	
	public ByteArray getNodeNameByteArray() {
		byte[] byteNodeName = ByteConversion.string2UTF8(myNodeName);
		byte[] data = new byte[4 + byteNodeName.length];

		ByteConversion.intToByte(byteNodeName.length, data, 0);
		System.arraycopy(byteNodeName, 0, data, 4, byteNodeName.length);

		return new FixedByteArray(data);		
	}

	public Value getPreorderKey() {
		byte[] data = new byte[4];
		ByteConversion.intToByte(myPreorder, data, 0);
		return new Value(data);
	}

	public Value getPostorderKey() {
		byte[] data = new byte[4];
		ByteConversion.intToByte(myPostorder, data, 0);
		return new Value(data);
	}

	public Value getParentKey() {
		byte[] data = new byte[4];
		ByteConversion.intToByte(myParent, data, 0);
		return new Value(data);
	}

	@Override
	public String toString() {
		return "(pre: " + myPreorder + ", post: " + myPostorder + ", par: "
				+ myParent + ", att: " + myIsAttribute + ", tag: " + myNodeName
				+ ")";
	}
}
