package org.kropp.tests;

import java.util.List;

import org.kropp.index.Index;
import org.kropp.index.impl.SearchEngine;
import org.kropp.util.xml.XMLIndexFactory;
import org.kropp.util.xml.XPathQuery;

import junit.framework.TestCase;

public class SearchEngineTestShakespeareXml extends TestCase {
	private Index myTestIndex;
	
	public void setUp() {
		myTestIndex = XMLIndexFactory.getInstance().createIndex("shakespeare.xml");
	}
	
	public void tearDown() {
		myTestIndex.delete();
	}

	public void testSearch1() {
		List<String> result = SearchEngine.search(myTestIndex, new XPathQuery("//PLAY/TITLE"));
		
		assertEquals("[A Midsummer Night's Dream]", result.toString());
	}
	
	public void testSearch2() {
		List<String> result = SearchEngine.search(myTestIndex, new XPathQuery("//PLAY/PLAYSUBT"));
		
		assertEquals("[A MIDSUMMER NIGHT'S DREAM]", result.toString());
	}
	
	public void testSearch3() {
		List<String> result = SearchEngine.search(myTestIndex, new XPathQuery("//PLAY/PERSONAE/PGROUP/PERSONA"));
		
		assertEquals("[LYSANDER, DEMETRIUS, PEASEBLOSSOM, COBWEB, MOTH, MUSTARDSEED]", result.toString());
	}
	
	public void testSearch4() {
		List<String> result = SearchEngine.search(myTestIndex, new XPathQuery("//PLAY/PERSONAE/TITLE/following-sibling::PERSONA"));
		
		assertEquals("[THESEUS, Duke of Athens., EGEUS, father to Hermia., PHILOSTRATE, master of the revels to Theseus., QUINCE, a carpenter., SNUG, a joiner., BOTTOM, a weaver., FLUTE, a bellows-mender., SNOUT, a tinker., STARVELING, a tailor., HIPPOLYTA, queen of the Amazons, betrothed to Theseus., HERMIA, daughter to Egeus, in love with Lysander., HELENA, in love with Demetrius., OBERON, king of the fairies., TITANIA, queen of the fairies., PUCK, or Robin Goodfellow., Other fairies attending their King and Queen., Attendants on Theseus and Hippolyta.]", result.toString());
	}
	
	public void testSearch5() {
		List<String> result = SearchEngine.search(myTestIndex, new XPathQuery("//PLAY/PERSONAE/PGROUP/GRPDESCR/preceding-sibling::PERSONA"));
		
		assertEquals("[LYSANDER, DEMETRIUS, PEASEBLOSSOM, COBWEB, MOTH, MUSTARDSEED]", result.toString());
	}
	
	public void testSearch6() {
		List<String> result = SearchEngine.search(myTestIndex, new XPathQuery("//PLAY/FM/P/parent::FM"));
		
		assertEquals("[\n\n\n\n\n]", result.toString());
	}
}
