package org.kropp.util;

import java.util.*;

/**
 * @author Victor Kropp
 */
public class BTree<K extends Comparable<K>, V> implements Iterable<V> {
	public BTree() {
		myRoot = new BTreeNode<K, V>(ORDER);
	}

	public void put(K key, V value) {
		myRoot.put(key, value);

		while (myRoot.getParent() != null) {
			myRoot = myRoot.getParent();
		}
	}

	public V getValue(K key) {
		return myRoot.getValue(key);
	}

	@Override
	public String toString() {
		return myRoot.toString();
	}

	public Iterator<V> iterator() {
		return new BTreeIterator();
	}

	private class BTreeIterator implements Iterator<V> {
		private BTreeLeafNode<K, V> myCurrent;

		private Iterator<V> myIterator;

		private BTreeIterator() {
			BTreeNode<K, V> current = myRoot;
			while (current.getFirstChild() != null) {
				AbstractBTreeNode<K, V> next = current.getFirstChild();
				if (next instanceof BTreeLeafNode) {
					myCurrent = (BTreeLeafNode<K, V>) next;
					break;
				} else {
					current = (BTreeNode<K, V>) next;
				}
			}

			if (myCurrent != null) {
				myIterator = myCurrent.iterator();
			}
		}

		public boolean hasNext() {
			return (myCurrent != null)
					&& (myIterator.hasNext() || (myCurrent.getNext() != null));
		}

		public V next() {
			if (myIterator.hasNext()) {
				return myIterator.next();
			} else {
				if (myCurrent.getNext() != null) {
					myCurrent = myCurrent.getNext();
					myIterator = myCurrent.iterator();
					if (myIterator.hasNext()) {
						return myIterator.next();
					}
				}
			}
			return null;
		}

		public void remove() {
			throw new RuntimeException();
		}
	}

	/**
	 * Tree order. (Maximum number of children of all nodes. Leafs have ORDER+1
	 * values.)
	 */
	private static final int ORDER = 4;

	private BTreeNode<K, V> myRoot;
}
