package org.kropp.util;

import java.util.ArrayList;

class BTreeNode<K extends Comparable<K>, V> extends AbstractBTreeNode<K, V> {
	public BTreeNode(int order) {
		this(order, null);
	}

	public BTreeNode(int order, AbstractBTreeNode<K, V> parent) {
		super(order, parent);
		myChildren = new ArrayList<AbstractBTreeNode<K, V>>(order + 1);
	}

	@Override
	public void put(K key, V value) {
		if (myKeys.size() == 0) {
			BTreeLeafNode<K, V> newLeaf = new BTreeLeafNode<K, V>(myOrder, this);
			myKeys.add(key);
			myChildren.add(newLeaf);
			newLeaf.put(key, value);
		} else {
			int index = indexOf(key);
			while ((index < myKeys.size())
					&& (myKeys.get(index).compareTo(key) < 0)) {
				index++;
			}
			if ((index == myChildren.size())
					&& (myChildren.size() < myOrder + 1)) {
				index--;
			}
			if (index < myChildren.size()) {
				myChildren.get(index).put(key, value);
			} else {
				BTreeLeafNode<K, V> newLeaf = new BTreeLeafNode<K, V>(myOrder,
						this);
				myKeys.add(key);
				myChildren.add(newLeaf);
				newLeaf.put(key, value);
			}
		}
	}

	void addNode(K key, AbstractBTreeNode<K, V> node) {
		int index = indexOf(key);
		if (index < 0) {
			index = 0;
		}
		while ((index < myChildren.size())
				&& (myChildren.get(index).getMaxKey().compareTo(key) < 0)) {
			index++;
		}

		if (myKeys.size() < myOrder) {
			if (index < myKeys.size()) {
				myKeys.add(index, key);
			} else {
				myKeys.add(key);
			}
		}
		myChildren.add(index, node);

		if (myChildren.size() > myOrder + 1) {
			/*
			 * Fixing keys.
			 */
			myKeys.add(null);
			for (int i = 0; i <= myOrder; i++) {
				myKeys.set(i, myChildren.get(i).getMaxKey());
			}

			//System.err.println("was:    " + this + "=]=]");
			BTreeNode<K, V> newNode = new BTreeNode<K, V>(myOrder, myParent);
			for (int i = myKeys.size() - 1; i > myOrder / 2; i--) {
				newNode.myKeys.add(0, myKeys.remove(i));
				newNode.myChildren.add(0, myChildren.remove(i + 1));
			}
			/*
			 * Fixing parent references in newNode.
			 */
			for (AbstractBTreeNode<K, V> n : newNode.myChildren) {
				n.myParent = newNode;
			}
			newNode.myChildren.add(0, myChildren.remove(myOrder / 2 + 1));
			//System.err.println("newnnn::::" + newNode + "]]]]");

			if (myParent != null) {
				((BTreeNode<K, V>) myParent).updateKeyFor(this, myKeys
						.remove(myOrder / 2));
				myParent.addNode(newNode.getMaxKey(), newNode);
			} else {
				BTreeNode<K, V> parent = new BTreeNode<K, V>(myOrder, null);
				parent.myKeys.add(myKeys.remove(myOrder / 2));
				parent.myChildren.add(this);
				parent.myChildren.add(newNode);
				newNode.myParent = parent;
				myParent = parent;
			}
		}
	}

	public BTreeNode<K, V> getParent() {
		return (BTreeNode<K, V>) myParent;
	}

	@Override
	public V getValue(K key) {
		int index = indexOf(key);
		if (index < 0) {
			index = 0;
		}
		while ((index < myChildren.size())
				&& (myChildren.get(index).getMaxKey().compareTo(key) < 0)) {
			index++;
		}
		return myChildren.get(index).getValue(key);
	}

	@Override
	public String toString() {
		String result = super.toString() + "\nChildren: {";
		for (AbstractBTreeNode<K, V> node : myChildren) {
			result += node + " ";
		}
		return result + "}";
	}

	void updateKeyFor(AbstractBTreeNode<K, V> node, K key) {
		for (int i = 0; i < myChildren.size(); i++) {
			Object current = myChildren.get(i);
			if (current instanceof AbstractBTreeNode) {
				AbstractBTreeNode cur = (AbstractBTreeNode) current;
				if (cur == node) {
					if (i < myOrder) {
						if (i < myKeys.size()) {
							myKeys.set(i, key);
						}
						/*
						 * if ((i == myChildren.size() - 1) && (myParent !=
						 * null)) { ((BTreeNode<K, V>) myParent)
						 * .updateKeyFor(this, key); }
						 */
					}
					return;
				}
			}
		}
		// throw new NoSuchElementException();
	}

	private ArrayList<AbstractBTreeNode<K, V>> myChildren;

	@Override
	K getMaxKey() {
		if (myChildren.size() == 0) {
			return null;
		} else {
			return myChildren.get(myChildren.size() - 1).getMaxKey();
		}
	}
	
	public AbstractBTreeNode<K, V> getFirstChild() {
		if(myChildren.size() == 0) {
			return null;
		} else {
			return myChildren.get(0);
		}
	}
}
