package org.kropp.util.xml;

import java.io.*;

import javax.xml.parsers.*;

import net.sf.xanswer.indexing.IndexException;

import org.kropp.index.Index;
import org.kropp.index.IndexFiller;
import org.kropp.index.impl.IndexImpl;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * @author Victor Kropp
 */
public class XMLIndexFactory {
	private static XMLIndexFactory myInstance = null;

	private XMLReader myReader;

	/**
	 * Singleton
	 */
	private XMLIndexFactory() {
		try {
			SAXParserFactory factory = SAXParserFactory.newInstance();

			SAXParser parser = factory.newSAXParser();
			myReader = parser.getXMLReader();
		} catch (SAXException e) {
			System.out.println("Cannot parse: " + e.getMessage());
		} catch (ParserConfigurationException e) {
			System.out.println("Cannot create parser: " + e.getMessage());
		}
	}

	public static XMLIndexFactory getInstance() {
		if (myInstance == null) {
			myInstance = new XMLIndexFactory();
		}
		return myInstance;
	}

	public Index createIndex(String filename) {
		return createIndex(filename, "");
	}

	public Index createIndex(String filename, String directory) {
		Index index = new IndexImpl(filename, directory);
		try {
			index.create(filename);
			myReader.setContentHandler(new IndexContentHandler(index));

			InputStream is = new FileInputStream(filename);
			try {
				myReader.parse(new InputSource(is));
			} finally {
				is.close();
			}
		} catch (IndexException e) {
			System.out.println("Cannot create index: " + e.getMessage());
		} catch (SAXException e) {
			System.out.println("Cannot parse file: " + e.getMessage());
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
			System.out.println("Cannot read file: " + filename + ": "
					+ e.getMessage());
		}

		index.flush();

		return index;
	}

	public Index openIndex(String originalFilename, String directory)
			throws IndexException {
		Index index = new IndexImpl(originalFilename, directory);

		index.open();

		return index;
	}

	private static class IndexContentHandler extends DefaultHandler {
		private IndexFiller myFiller;

		public IndexContentHandler(Index index) {
			myFiller = new IndexFiller(index);
		}

		@Override
		public void startElement(String uri, String localName, String qName,
				Attributes attributes) {
			myFiller.startElement(qName, "");

			for (int i = 0; i < attributes.getLength(); i++) {
				myFiller.startElement(attributes.getQName(i), attributes
						.getValue(i), true);
				myFiller.endElement();
			}
		}

		@Override
		public void endElement(String uri, String localName, String qName)
				throws SAXException {
			myFiller.endElement();
		}

		@Override
		public void characters(char[] ch, int start, int length)
				throws SAXException {
			myFiller.appendElementValue(new String(ch, start, length));
		}
	}
}
